@@ -1,7 +1,7 @@
 /* Miscellaneous functions, not really specific to GNU tar.
 
-   Copyright 1988, 1992, 1994, 1995, 1996, 1997, 1999, 2000, 2001 Free
-   Software Foundation, Inc.
+   Copyright (C) 1988, 1992, 1994, 1995, 1996, 1997, 1999, 2000, 2001,
+   2003, 2004 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -23,7 +23,7 @@
 #include <quotearg.h>
 #include <save-cwd.h>
 
-static void call_arg_fatal PARAMS ((char const *, char const *))
+static void call_arg_fatal (char const *, char const *)
      __attribute__ ((noreturn));
 
 /* Handling strings.  */
@@ -96,7 +96,7 @@ quote_copy_string (const char *string)
    and turns it back into the un-quoted original.  This is done in
    place.  Returns 0 only if the string was not properly quoted, but
    completes the unquoting anyway.
- 									   
+
    This is used for reading the saved directory file in incremental
    dumps.  It is used for decoding old `N' records (demangling names).
    But also, it is used for decoding file arguments, would they come
@@ -197,47 +197,48 @@ unquote_string (char *string)
   return result;
 }
 
-/* Return nonzero if NAME contains ".." as a path name component.  */
-int
-contains_dot_dot (char const *name)
-{
-  char const *p = name + FILESYSTEM_PREFIX_LEN (name);
-
-  for (;;)
-    {
-      if (p[0] == '.' && p[1] == '.' && (ISSLASH (p[2]) || !p[2]))
-	return 1;
-
-      do
-	{
-	  if (! *p++)
-	    return 0;
-	}
-      while (! ISSLASH (*p));
-    }
-}
-
 /* File handling.  */
 
 /* Saved names in case backup needs to be undone.  */
 static char *before_backup_name;
 static char *after_backup_name;
 
-/* Some implementations of rmdir let you remove the working directory.
+/* Return 1 if PATH is obviously "." or "/".  */
+static bool
+must_be_dot_or_slash (char const *path)
+{
+  path += FILESYSTEM_PREFIX_LEN (path);
+
+  if (ISSLASH (path[0]))
+    {
+      for (;;)
+	if (ISSLASH (path[1]))
+	  path++;
+	else if (path[1] == '.' && ISSLASH (path[2 + (path[2] == '.')]))
+	  path += 2 + (path[2] == '.');
+	else
+	  return ! path[1];
+    }
+  else
+    {
+      while (path[0] == '.' && ISSLASH (path[1]))
+	{
+	  path += 2;
+	  while (ISSLASH (*path))
+	    path++;
+	}
+
+      return ! path[0] || (path[0] == '.' && ! path[1]);
+    }
+}
+
+/* Some implementations of rmdir let you remove '.' or '/'.
    Report an error with errno set to zero for obvious cases of this;
    otherwise call rmdir.  */
 static int
 safer_rmdir (const char *path)
 {
-  while (path[0] == '.' && ISSLASH (path[1]))
-    {
-      path++;
-
-      do path++;
-      while (ISSLASH (*path));
-    }
-
-  if (! path[0] || (path[0] == '.' && ! path[1]))
+  if (must_be_dot_or_slash (path))
     {
       errno = 0;
       return -1;
@@ -246,12 +247,15 @@ safer_rmdir (const char *path)
   return rmdir (path);
 }
 
-/* Remove PATH.  If PATH is a directory, then if RECURSE is set remove
-   it recursively; otherwise, remove it only if it is empty.  Return 0
-   on error, with errno set; if PATH is obviously the working
+/* Remove PATH, returning 1 on success.  If PATH is a directory, then
+   if OPTION is RECURSIVE_REMOVE_OPTION is set remove PATH
+   recursively; otherwise, remove it only if it is empty.  If PATH is
+   a directory that cannot be removed (e.g., because it is nonempty)
+   and if OPTION is WANT_DIRECTORY_REMOVE_OPTION, then return -1.
+   Return 0 on error, with errno set; if PATH is obviously the working
    directory return zero with errno set to zero.  */
 int
-remove_any_file (const char *path, int recurse)
+remove_any_file (const char *path, enum remove_option option)
 {
   /* Try unlink first if we are not root, as this saves us a system
      call in the common case where we're removing a non-directory.  */
@@ -259,7 +263,11 @@ remove_any_file (const char *path, int recurse)
     {
       if (unlink (path) == 0)
 	return 1;
-      if (errno != EPERM)
+
+      /* POSIX 1003.1-2001 requires EPERM when attempting to unlink a
+	 directory without appropriate privileges, but many Linux
+	 kernels return the more-sensible EISDIR.  */
+      if (errno != EPERM && errno != EISDIR)
 	return 0;
     }
 
@@ -276,34 +284,43 @@ remove_any_file (const char *path, int recurse)
 #if defined ENOTEMPTY && ENOTEMPTY != EEXIST
     case ENOTEMPTY:
 #endif
-      if (recurse)
+      switch (option)
 	{
-	  char *directory = savedir (path);
-	  char const *entry;
-	  size_t entrylen;
+	case ORDINARY_REMOVE_OPTION:
+	  break;
 
-	  if (! directory)
-	    return 0;
+	case WANT_DIRECTORY_REMOVE_OPTION:
+	  return -1;
 
-	  for (entry = directory;
-	       (entrylen = strlen (entry)) != 0;
-	       entry += entrylen + 1)
-	    {
-	      char *path_buffer = new_name (path, entry);
-	      int r = remove_any_file (path_buffer, 1);
-	      int e = errno;
-	      free (path_buffer);
+	case RECURSIVE_REMOVE_OPTION:
+	  {
+	    char *directory = savedir (path);
+	    char const *entry;
+	    size_t entrylen;
 
-	      if (! r)
-		{
-		  free (directory);
-		  errno = e;
-		  return 0;
-		}
-	    }
+	    if (! directory)
+	      return 0;
 
-	  free (directory);
-	  return safer_rmdir (path) == 0;
+	    for (entry = directory;
+		 (entrylen = strlen (entry)) != 0;
+		 entry += entrylen + 1)
+	      {
+		char *path_buffer = new_name (path, entry);
+		int r = remove_any_file (path_buffer, 1);
+		int e = errno;
+		free (path_buffer);
+
+		if (! r)
+		  {
+		    free (directory);
+		    errno = e;
+		    return 0;
+		  }
+	      }
+
+	    free (directory);
+	    return safer_rmdir (path) == 0;
+	  }
 	}
       break;
     }
@@ -312,34 +329,36 @@ remove_any_file (const char *path, int recurse)
 }
 
 /* Check if PATH already exists and make a backup of it right now.
-   Return success (nonzero) only if the backup in either unneeded, or
+   Return success (nonzero) only if the backup is either unneeded, or
    successful.  For now, directories are considered to never need
-   backup.  If ARCHIVE is nonzero, this is the archive and so, we do
-   not have to backup block or character devices, nor remote entities.  */
-int
-maybe_backup_file (const char *path, int archive)
+   backup.  If THIS_IS_THE_ARCHIVE is nonzero, this is the archive and
+   so, we do not have to backup block or character devices, nor remote
+   entities.  */
+bool
+maybe_backup_file (const char *path, int this_is_the_archive)
 {
   struct stat file_stat;
 
   /* Check if we really need to backup the file.  */
 
-  if (archive && _remdev (path))
-    return 1;
+  if (this_is_the_archive && _remdev (path))
+    return true;
 
   if (stat (path, &file_stat))
     {
       if (errno == ENOENT)
-	return 1;
+	return true;
 
       stat_error (path);
-      return 0;
+      return false;
     }
 
   if (S_ISDIR (file_stat.st_mode))
-    return 1;
+    return true;
 
-  if (archive && (S_ISBLK (file_stat.st_mode) || S_ISCHR (file_stat.st_mode)))
-    return 1;
+  if (this_is_the_archive
+      && (S_ISBLK (file_stat.st_mode) || S_ISCHR (file_stat.st_mode)))
+    return true;
 
   assign_string (&before_backup_name, path);
 
@@ -359,7 +378,7 @@ maybe_backup_file (const char *path, int archive)
 	fprintf (stdlis, _("Renaming %s to %s\n"),
 		 quote_n (0, before_backup_name),
 		 quote_n (1, after_backup_name));
-      return 1;
+      return true;
     }
   else
     {
@@ -369,7 +388,7 @@ maybe_backup_file (const char *path, int archive)
 	      quotearg_colon (before_backup_name),
 	      quote_n (1, after_backup_name)));
       assign_string (&after_backup_name, 0);
-      return 0;
+      return false;
     }
 }
 
@@ -397,7 +416,7 @@ undo_last_backup (void)
 
 /* Depending on DEREF, apply either stat or lstat to (NAME, BUF).  */
 int
-deref_stat (int deref, char const *name, struct stat *buf)
+deref_stat (bool deref, char const *name, struct stat *buf)
 {
   return deref ? stat (name, buf) : lstat (name, buf);
 }
@@ -473,7 +492,7 @@ chdir_do (int i)
 
       if (curr->saved)
 	{
-	  if (restore_cwd (&curr->saved_cwd, curr->name, prev->name))
+	  if (restore_cwd (&curr->saved_cwd))
 	    FATAL_ERROR ((0, 0, _("Cannot change working directory")));
 	}
       else
@@ -569,18 +588,21 @@ close_error (char const *name)
   call_arg_error ("close", name);
 }
 
-void
-close_fatal (char const *name)
-{
-  call_arg_fatal ("close", name);
-}
-
 void
 close_warn (char const *name)
 {
   call_arg_warn ("close", name);
 }
 
+void
+close_diag (char const *name)
+{
+  if (ignore_failed_read_option)
+    close_warn (name);
+  else
+    close_error (name);
+}
+
 void
 exec_fatal (char const *name)
 {
@@ -631,6 +653,15 @@ open_warn (char const *name)
   call_arg_warn ("open", name);
 }
 
+void
+open_diag (char const *name)
+{
+  if (ignore_failed_read_option)
+    open_warn (name);
+  else
+    open_error (name);
+}
+
 void
 read_error (char const *name)
 {
@@ -643,7 +674,9 @@ read_error_details (char const *name, off_t offset, size_t size)
   char buf[UINTMAX_STRSIZE_BOUND];
   int e = errno;
   ERROR ((0, e,
-	  _("%s: Read error at byte %s, reading %lu bytes"),
+	  ngettext ("%s: Read error at byte %s, reading %lu byte",
+		    "%s: Read error at byte %s, reading %lu bytes",
+		    size),
 	  quotearg_colon (name), STRINGIFY_BIGINT (offset, buf),
 	  (unsigned long) size));
 }
@@ -654,11 +687,22 @@ read_warn_details (char const *name, off_t offset, size_t size)
   char buf[UINTMAX_STRSIZE_BOUND];
   int e = errno;
   WARN ((0, e,
-	 _("%s: Warning: Read error at byte %s, reading %lu bytes"),
+	 ngettext ("%s: Warning: Read error at byte %s, reading %lu byte",
+		   "%s: Warning: Read error at byte %s, reading %lu bytes",
+		   size),
 	 quotearg_colon (name), STRINGIFY_BIGINT (offset, buf),
 	 (unsigned long) size));
 }
 
+void
+read_diag_details (char const *name, off_t offset, size_t size)
+{
+  if (ignore_failed_read_option)
+    read_warn_details (name, offset, size);
+  else
+    read_error_details (name, offset, size);
+}
+
 void
 read_fatal (char const *name)
 {
@@ -671,7 +715,9 @@ read_fatal_details (char const *name, off_t offset, size_t size)
   char buf[UINTMAX_STRSIZE_BOUND];
   int e = errno;
   FATAL_ERROR ((0, e,
-		_("%s: Read error at byte %s, reading %lu bytes"),
+		ngettext ("%s: Read error at byte %s, reading %lu byte",
+			  "%s: Read error at byte %s, reading %lu bytes",
+			  size),
 		quotearg_colon (name), STRINGIFY_BIGINT (offset, buf),
 		(unsigned long) size));
 }
@@ -688,6 +734,15 @@ readlink_warn (char const *name)
   call_arg_warn ("readlink", name);
 }
 
+void
+readlink_diag (char const *name)
+{
+  if (ignore_failed_read_option)
+    readlink_warn (name);
+  else
+    readlink_error (name);
+}
+
 void
 savedir_error (char const *name)
 {
@@ -700,6 +755,15 @@ savedir_warn (char const *name)
   call_arg_warn ("savedir", name);
 }
 
+void
+savedir_diag (char const *name)
+{
+  if (ignore_failed_read_option)
+    savedir_warn (name);
+  else
+    savedir_error (name);
+}
+
 void
 seek_error (char const *name)
 {
@@ -732,6 +796,15 @@ seek_warn_details (char const *name, off_t offset)
 	 STRINGIFY_BIGINT (offset, buf)));
 }
 
+void
+seek_diag_details (char const *name, off_t offset)
+{
+  if (ignore_failed_read_option)
+    seek_warn_details (name, offset);
+  else
+    seek_error_details (name, offset);
+}
+
 void
 symlink_error (char const *contents, char const *name)
 {
@@ -752,6 +825,15 @@ stat_warn (char const *name)
   call_arg_warn ("stat", name);
 }
 
+void
+stat_diag (char const *name)
+{
+  if (ignore_failed_read_option)
+    stat_warn (name);
+  else
+    stat_error (name);
+}
+
 void
 truncate_error (char const *name)
 {
@@ -789,13 +871,16 @@ write_error (char const *name)
 }
 
 void
-write_error_details (char const *name, ssize_t status, size_t size)
+write_error_details (char const *name, size_t status, size_t size)
 {
-  if (status < 0)
+  if (status == 0)
     write_error (name);
   else
-    ERROR ((0, 0, _("%s: Wrote only %lu of %lu bytes"),
-	    name, (unsigned long) status, (unsigned long) record_size));
+    ERROR ((0, 0,
+	    ngettext ("%s: Wrote only %lu of %lu byte",
+		      "%s: Wrote only %lu of %lu bytes",
+		      size),
+	    name, (unsigned long int) status, (unsigned long int) size));
 }
 
 void
