@@ -1,7 +1,7 @@
 /* A tar (tape archiver) program.
 
-   Copyright (C) 1988, 1992, 1993, 1994, 1995, 1996, 1997, 1999, 2000, 2001
-   Free Software Foundation, Inc.
+   Copyright (C) 1988, 1992, 1993, 1994, 1995, 1996, 1997, 1999, 2000,
+   2001, 2003, 2004 Free Software Foundation, Inc.
 
    Written by John Gilmore, starting 1985-08-25.
 
@@ -36,16 +36,18 @@
 #define GLOBAL
 #include "common.h"
 
-#include <print-copyr.h>
+#include <getdate.h>
 #include <localedir.h>
 #include <prepargs.h>
 #include <quotearg.h>
 #include <xstrtol.h>
 
-time_t get_date ();
-
 /* Local declarations.  */
 
+#ifndef DEFAULT_ARCHIVE_FORMAT
+# define DEFAULT_ARCHIVE_FORMAT GNU_FORMAT
+#endif
+
 #ifndef DEFAULT_ARCHIVE
 # define DEFAULT_ARCHIVE "tar.out"
 #endif
@@ -54,7 +56,6 @@ time_t get_date ();
 # define DEFAULT_BLOCKING 20
 #endif
 
-static void usage PARAMS ((int)) __attribute__ ((noreturn));
 
 /* Miscellaneous.  */
 
@@ -114,6 +115,57 @@ confirm (const char *message_action, const char *message_name)
     return reply == 'y' || reply == 'Y';
   }
 }
+
+static struct fmttab {
+  char const *name;
+  enum archive_format fmt;
+} const fmttab[] = {
+  { "v7",      V7_FORMAT },
+  { "oldgnu",  OLDGNU_FORMAT },
+  { "ustar",   USTAR_FORMAT },
+  { "posix",   POSIX_FORMAT },
+#if 0 /* not fully supported yet */
+  { "star",    STAR_FORMAT },
+#endif
+  { "gnu",     GNU_FORMAT },
+  { NULL,	 0 }
+};
+
+static void
+set_archive_format (char const *name)
+{
+  struct fmttab const *p;
+
+  for (p = fmttab; strcmp (p->name, name) != 0; )
+    if (! (++p)->name)
+      USAGE_ERROR ((0, 0, _("%s: Invalid archive format"),
+		    quotearg_colon (name)));
+
+  archive_format = p->fmt;
+}
+
+static const char *
+archive_format_string (enum archive_format fmt)
+{
+  struct fmttab const *p;
+
+  for (p = fmttab; p->name; p++)
+    if (p->fmt == fmt)
+      return p->name;
+  return "unknown?";
+}
+
+#define FORMAT_MASK(n) (1<<(n))
+
+static void
+assert_format(unsigned fmt_mask)
+{
+  if ((FORMAT_MASK(archive_format) & fmt_mask) == 0)
+    USAGE_ERROR ((0, 0,
+		  _("GNU features wanted on incompatible archive format")));
+}
+
+
 
 /* Options.  */
 
@@ -127,42 +179,48 @@ confirm (const char *message_action, const char *message_name)
 enum
 {
   ANCHORED_OPTION = CHAR_MAX + 1,
+  ATIME_PRESERVE_OPTION,
   BACKUP_OPTION,
+  CHECKPOINT_OPTION,
   DELETE_OPTION,
   EXCLUDE_OPTION,
+  FORCE_LOCAL_OPTION,
+  FORMAT_OPTION,
   GROUP_OPTION,
   IGNORE_CASE_OPTION,
+  IGNORE_FAILED_READ_OPTION,
+  INDEX_FILE_OPTION,
+  KEEP_NEWER_FILES_OPTION,
   MODE_OPTION,
   NEWER_MTIME_OPTION,
   NO_ANCHORED_OPTION,
   NO_IGNORE_CASE_OPTION,
+  NO_OVERWRITE_DIR_OPTION,
   NO_WILDCARDS_OPTION,
   NO_WILDCARDS_MATCH_SLASH_OPTION,
   NULL_OPTION,
+  NUMERIC_OWNER_OPTION,
+  OCCURRENCE_OPTION,
   OVERWRITE_OPTION,
-  OVERWRITE_DIR_OPTION,
   OWNER_OPTION,
+  PAX_OPTION,
   POSIX_OPTION,
   PRESERVE_OPTION,
   RECORD_SIZE_OPTION,
+  RECURSIVE_UNLINK_OPTION,
+  REMOVE_FILES_OPTION,
+  RMT_COMMAND_OPTION,
   RSH_COMMAND_OPTION,
+  SHOW_DEFAULTS_OPTION,
+  SHOW_OMITTED_DIRS_OPTION,
+  STRIP_PATH_OPTION,
   SUFFIX_OPTION,
+  TOTALS_OPTION,
   USE_COMPRESS_PROGRAM_OPTION,
+  UTC_OPTION,
   VOLNO_FILE_OPTION,
   WILDCARDS_OPTION,
-  WILDCARDS_MATCH_SLASH_OPTION,
-
-  /* Some cleanup is being made in GNU tar long options.  Using old names is
-     allowed for a while, but will also send a warning to stderr.  Take old
-     names out in 1.14, or in summer 1997, whichever happens last.  */
-
-  OBSOLETE_ABSOLUTE_NAMES,
-  OBSOLETE_BLOCK_COMPRESS,
-  OBSOLETE_BLOCKING_FACTOR,
-  OBSOLETE_BLOCK_NUMBER,
-  OBSOLETE_READ_FULL_RECORDS,
-  OBSOLETE_TOUCH,
-  OBSOLETE_VERSION_CONTROL
+  WILDCARDS_MATCH_SLASH_OPTION
 };
 
 /* If nonzero, display usage information and exit.  */
@@ -174,19 +232,17 @@ static int show_version;
 static struct option long_options[] =
 {
   {"absolute-names", no_argument, 0, 'P'},
-  {"absolute-paths", no_argument, 0, OBSOLETE_ABSOLUTE_NAMES},
   {"after-date", required_argument, 0, 'N'},
   {"anchored", no_argument, 0, ANCHORED_OPTION},
   {"append", no_argument, 0, 'r'},
-  {"atime-preserve", no_argument, &atime_preserve_option, 1},
+  {"atime-preserve", no_argument, 0, ATIME_PRESERVE_OPTION},
   {"backup", optional_argument, 0, BACKUP_OPTION},
-  {"block-compress", no_argument, 0, OBSOLETE_BLOCK_COMPRESS},
   {"block-number", no_argument, 0, 'R'},
-  {"block-size", required_argument, 0, OBSOLETE_BLOCKING_FACTOR},
   {"blocking-factor", required_argument, 0, 'b'},
   {"bzip2", no_argument, 0, 'j'},
   {"catenate", no_argument, 0, 'A'},
-  {"checkpoint", no_argument, &checkpoint_option, 1},
+  {"checkpoint", no_argument, 0, CHECKPOINT_OPTION},
+  {"check-links", no_argument, &check_links_option, 1},
   {"compare", no_argument, 0, 'd'},
   {"compress", no_argument, 0, 'Z'},
   {"concatenate", no_argument, 0, 'A'},
@@ -202,25 +258,27 @@ static struct option long_options[] =
   {"extract", no_argument, 0, 'x'},
   {"file", required_argument, 0, 'f'},
   {"files-from", required_argument, 0, 'T'},
-  {"force-local", no_argument, &force_local_option, 1},
+  {"force-local", no_argument, 0, FORCE_LOCAL_OPTION},
+  {"format", required_argument, 0, FORMAT_OPTION},
   {"get", no_argument, 0, 'x'},
   {"group", required_argument, 0, GROUP_OPTION},
   {"gunzip", no_argument, 0, 'z'},
   {"gzip", no_argument, 0, 'z'},
   {"help", no_argument, &show_help, 1},
   {"ignore-case", no_argument, 0, IGNORE_CASE_OPTION},
-  {"ignore-failed-read", no_argument, &ignore_failed_read_option, 1},
+  {"ignore-failed-read", no_argument, 0, IGNORE_FAILED_READ_OPTION},
   {"ignore-zeros", no_argument, 0, 'i'},
   /* FIXME: --ignore-end as a new name for --ignore-zeros?  */
   {"incremental", no_argument, 0, 'G'},
+  {"index-file", required_argument, 0, INDEX_FILE_OPTION},
   {"info-script", required_argument, 0, 'F'},
   {"interactive", no_argument, 0, 'w'},
+  {"keep-newer-files", no_argument, 0, KEEP_NEWER_FILES_OPTION},
   {"keep-old-files", no_argument, 0, 'k'},
   {"label", required_argument, 0, 'V'},
   {"list", no_argument, 0, 't'},
   {"listed-incremental", required_argument, 0, 'g'},
   {"mode", required_argument, 0, MODE_OPTION},
-  {"modification-time", no_argument, 0, OBSOLETE_TOUCH},
   {"multi-volume", no_argument, 0, 'M'},
   {"new-volume-script", required_argument, 0, 'F'},
   {"newer", required_argument, 0, 'N'},
@@ -228,51 +286,54 @@ static struct option long_options[] =
   {"null", no_argument, 0, NULL_OPTION},
   {"no-anchored", no_argument, 0, NO_ANCHORED_OPTION},
   {"no-ignore-case", no_argument, 0, NO_IGNORE_CASE_OPTION},
+  {"no-overwrite-dir", no_argument, 0, NO_OVERWRITE_DIR_OPTION},
   {"no-wildcards", no_argument, 0, NO_WILDCARDS_OPTION},
   {"no-wildcards-match-slash", no_argument, 0, NO_WILDCARDS_MATCH_SLASH_OPTION},
   {"no-recursion", no_argument, &recursion_option, 0},
   {"no-same-owner", no_argument, &same_owner_option, -1},
   {"no-same-permissions", no_argument, &same_permissions_option, -1},
-  {"numeric-owner", no_argument, &numeric_owner_option, 1},
+  {"numeric-owner", no_argument, 0, NUMERIC_OWNER_OPTION},
+  {"occurrence", optional_argument, 0, OCCURRENCE_OPTION},
   {"old-archive", no_argument, 0, 'o'},
   {"one-file-system", no_argument, 0, 'l'},
   {"overwrite", no_argument, 0, OVERWRITE_OPTION},
-  {"overwrite-dir", no_argument, 0, OVERWRITE_DIR_OPTION},
   {"owner", required_argument, 0, OWNER_OPTION},
+  {"pax-option", required_argument, 0, PAX_OPTION},
   {"portability", no_argument, 0, 'o'},
   {"posix", no_argument, 0, POSIX_OPTION},
   {"preserve", no_argument, 0, PRESERVE_OPTION},
   {"preserve-order", no_argument, 0, 's'},
   {"preserve-permissions", no_argument, 0, 'p'},
   {"recursion", no_argument, &recursion_option, FNM_LEADING_DIR},
-  {"recursive-unlink", no_argument, &recursive_unlink_option, 1},
-  {"read-full-blocks", no_argument, 0, OBSOLETE_READ_FULL_RECORDS},
+  {"recursive-unlink", no_argument, 0, RECURSIVE_UNLINK_OPTION},
   {"read-full-records", no_argument, 0, 'B'},
   /* FIXME: --partial-blocks might be a synonym for --read-full-records?  */
-  {"record-number", no_argument, 0, OBSOLETE_BLOCK_NUMBER},
   {"record-size", required_argument, 0, RECORD_SIZE_OPTION},
-  {"remove-files", no_argument, &remove_files_option, 1},
+  {"remove-files", no_argument, 0, REMOVE_FILES_OPTION},
+  {"rmt-command", required_argument, 0, RMT_COMMAND_OPTION},
   {"rsh-command", required_argument, 0, RSH_COMMAND_OPTION},
   {"same-order", no_argument, 0, 's'},
   {"same-owner", no_argument, &same_owner_option, 1},
   {"same-permissions", no_argument, 0, 'p'},
-  {"show-omitted-dirs", no_argument, &show_omitted_dirs_option, 1},
+  {"show-defaults", no_argument, 0, SHOW_DEFAULTS_OPTION},
+  {"show-omitted-dirs", no_argument, 0, SHOW_OMITTED_DIRS_OPTION},
   {"sparse", no_argument, 0, 'S'},
   {"starting-file", required_argument, 0, 'K'},
+  {"strip-path", required_argument, 0, STRIP_PATH_OPTION },
   {"suffix", required_argument, 0, SUFFIX_OPTION},
   {"tape-length", required_argument, 0, 'L'},
   {"to-stdout", no_argument, 0, 'O'},
-  {"totals", no_argument, &totals_option, 1},
+  {"totals", no_argument, 0, TOTALS_OPTION},
   {"touch", no_argument, 0, 'm'},
   {"uncompress", no_argument, 0, 'Z'},
   {"ungzip", no_argument, 0, 'z'},
   {"unlink-first", no_argument, 0, 'U'},
   {"update", no_argument, 0, 'u'},
+  {"utc", no_argument, 0, UTC_OPTION },
   {"use-compress-program", required_argument, 0, USE_COMPRESS_PROGRAM_OPTION},
   {"verbose", no_argument, 0, 'v'},
   {"verify", no_argument, 0, 'W'},
   {"version", no_argument, &show_version, 1},
-  {"version-control", required_argument, 0, OBSOLETE_VERSION_CONTROL},
   {"volno-file", required_argument, 0, VOLNO_FILE_OPTION},
   {"wildcards", no_argument, 0, WILDCARDS_OPTION},
   {"wildcards-match-slash", no_argument, 0, WILDCARDS_MATCH_SLASH_OPTION},
@@ -281,7 +342,7 @@ static struct option long_options[] =
 };
 
 /* Print a usage message and exit with STATUS.  */
-static void
+void
 usage (int status)
 {
   if (status != TAREXIT_SUCCESS)
@@ -324,8 +385,10 @@ Operation modifiers:\n\
   -W, --verify               attempt to verify the archive after writing it\n\
       --remove-files         remove files after adding them to the archive\n\
   -k, --keep-old-files       don't replace existing files when extracting\n\
+      --keep-newer-files     don't replace existing files that are newer\n\
+                             than their archive copies\n\
       --overwrite            overwrite existing files when extracting\n\
-      --overwrite-dir        overwrite directory metadata when extracting\n\
+      --no-overwrite-dir     preserve metadata of existing directories\n\
   -U, --unlink-first         remove each file prior to extracting over it\n\
       --recursive-unlink     empty hierarchies prior to extracting directory\n\
   -S, --sparse               handle sparse files efficiently\n\
@@ -333,7 +396,14 @@ Operation modifiers:\n\
   -G, --incremental          handle old GNU-format incremental backup\n\
   -g, --listed-incremental=FILE\n\
                              handle new GNU-format incremental backup\n\
-      --ignore-failed-read   do not exit with nonzero on unreadable files\n"),
+      --ignore-failed-read   do not exit with nonzero on unreadable files\n\
+      --occurrence[=NUM]     process only the NUMth occurrence of each file in\n\
+                             the archive. This option is valid only in\n\
+                             conjunction with one of the subcommands --delete,\n\
+                             --diff, --extract or --list and when a list of\n\
+                             files is given either on the command line or\n\
+                             via -T option.\n\
+                             NUM defaults to 1.\n"),
 	     stdout);
       fputs (_("\
 \n\
@@ -358,6 +428,7 @@ Handling of file attributes:\n\
 Device selection and switching:\n\
   -f, --file=ARCHIVE             use archive file or device ARCHIVE\n\
       --force-local              archive file is local even if has a colon\n\
+      --rmt-command=COMMAND      use given rmt COMMAND instead of /etc/rmt\n\
       --rsh-command=COMMAND      use remote COMMAND instead of rsh\n\
   -[0-7][lmh]                    specify drive and density\n\
   -M, --multi-volume             create/list/extract multi-volume archive\n\
@@ -377,10 +448,19 @@ Device blocking:\n\
       fputs (_("\
 \n\
 Archive format selection:\n\
+      --format=FMTNAME               create archive of the given format.\n\
+                                     FMTNAME is one of the following:\n\
+                                     v7        old V7 tar format\n\
+                                     oldgnu    GNU format as per tar <= 1.12\n\
+                                     gnu       GNU tar 1.13 format\n\
+                                     ustar     POSIX 1003.1-1988 (ustar) format\n\
+                                     posix     POSIX 1003.1-2001 (pax) format\n\
+      --old-archive, --portability   same as --format=v7\n\
+      --posix                        same as --format=posix\n\
+  --pax-option keyword[[:]=value][,keyword[[:]=value], ...]\n\
+                                     control pax keywords\n\
   -V, --label=NAME                   create archive with volume name NAME\n\
               PATTERN                at list/extract time, a globbing PATTERN\n\
-  -o, --old-archive, --portability   write a V7 format archive\n\
-      --posix                        write a POSIX format archive\n\
   -j, --bzip2                        filter the archive through bzip2\n\
   -z, --gzip, --ungzip               filter the archive through gzip\n\
   -Z, --compress, --uncompress       filter the archive through compress\n\
@@ -406,11 +486,13 @@ Local file selection:\n\
   -h, --dereference            dump instead the files symlinks point to\n\
       --no-recursion           avoid descending automatically in directories\n\
   -l, --one-file-system        stay in local file system when creating archive\n\
-  -K, --starting-file=NAME     begin at file NAME in the archive\n"),
+  -K, --starting-file=NAME     begin at file NAME in the archive\n\
+      --strip-path=NUM         strip NUM leading components from file names\n\
+                               before extraction\n"),
 	     stdout);
 #if !MSDOS
       fputs (_("\
-  -N, --newer=DATE             only store files newer than DATE\n\
+  -N, --newer=DATE-OR-FILE     only store files newer than DATE-OR-FILE\n\
       --newer-mtime=DATE       compare date and time when data changed only\n\
       --after-date=DATE        same as -N\n"),
 	     stdout);
@@ -426,13 +508,23 @@ Informative output:\n\
       --version         print tar program version number, then exit\n\
   -v, --verbose         verbosely list files processed\n\
       --checkpoint      print directory names while reading the archive\n\
+      --check-links     print a message if not all links are dumped\n\
       --totals          print total bytes written while creating archive\n\
+      --index-file=FILE send verbose output to FILE\n\
+      --utc             print file modification dates in UTC\n\
   -R, --block-number    show block number within archive with each message\n\
   -w, --interactive     ask for confirmation for every action\n\
       --confirmation    same as -w\n"),
 	     stdout);
       fputs (_("\
 \n\
+Compatibility options:\n\
+  -o                                 when creating, same as --old-archive\n\
+                                     when extracting, same as --no-same-owner\n"),
+             stdout);
+
+      fputs (_("\
+\n\
 The backup suffix is `~', unless set with --suffix or SIMPLE_BACKUP_SUFFIX.\n\
 The version control may be set with --backup or VERSION_CONTROL, values are:\n\
 \n\
@@ -442,14 +534,12 @@ The version control may be set with --backup or VERSION_CONTROL, values are:\n\
 	     stdout);
       printf (_("\
 \n\
-GNU tar cannot read nor produce `--posix' archives.  If POSIXLY_CORRECT\n\
-is set in the environment, GNU extensions are disallowed with `--posix'.\n\
-Support for POSIX is only partially implemented, don't count on it yet.\n\
 ARCHIVE may be FILE, HOST:FILE or USER@HOST:FILE; DATE may be a textual date\n\
 or a file name starting with `/' or `.', in which case the file's date is used.\n\
-*This* `tar' defaults to `-f%s -b%d'.\n"),
+*This* `tar' defaults to `--format=%s -f%s -b%d'.\n"),
+	      archive_format_string (DEFAULT_ARCHIVE_FORMAT),
 	      DEFAULT_ARCHIVE, DEFAULT_BLOCKING);
-      fputs (_("\nReport bugs to <bug-tar@gnu.org>.\n"), stdout);
+      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
     }
   exit (status);
 }
@@ -494,9 +584,12 @@ decode_options (int argc, char **argv)
 {
   int optchar;			/* option letter */
   int input_files;		/* number of input files */
-  const char *backup_suffix_string;
-  const char *version_control_string = 0;
+  char const *textual_date_option = 0;
+  char const *backup_suffix_string;
+  char const *version_control_string = 0;
   int exclude_options = EXCLUDE_WILDCARDS;
+  bool o_option = 0;
+  int pax_option = 0;
 
   /* Set some default option values.  */
 
@@ -505,7 +598,8 @@ decode_options (int argc, char **argv)
   blocking_factor = DEFAULT_BLOCKING;
   record_size = DEFAULT_BLOCKING * BLOCKSIZE;
   excluded = new_exclude ();
-  newer_mtime_option = TYPE_MINIMUM (time_t);
+  newer_mtime_option.tv_sec = TYPE_MINIMUM (time_t);
+  newer_mtime_option.tv_nsec = -1;
   recursion_option = FNM_LEADING_DIR;
 
   owner_option = -1;
@@ -534,7 +628,7 @@ decode_options (int argc, char **argv)
       /* Allocate a new argument array, and copy program name in it.  */
 
       new_argc = argc - 1 + strlen (argv[1]);
-      new_argv = xmalloc (new_argc * sizeof (char *));
+      new_argv = xmalloc ((new_argc + 1) * sizeof (char *));
       in = argv;
       out = new_argv;
       *out++ = *in++;
@@ -561,6 +655,7 @@ decode_options (int argc, char **argv)
 
       while (in < argv + argc)
 	*out++ = *in++;
+      *out = 0;
 
       /* Replace the old option list by the new one.  */
 
@@ -596,14 +691,6 @@ decode_options (int argc, char **argv)
 	set_subcommand_option (CAT_SUBCOMMAND);
 	break;
 
-      case OBSOLETE_BLOCK_COMPRESS:
-	WARN ((0, 0, _("Obsolete option, now implied by --blocking-factor")));
-	break;
-
-      case OBSOLETE_BLOCKING_FACTOR:
-	WARN ((0, 0, _("Obsolete option name replaced by --blocking-factor")));
-	/* Fall through.  */
-
       case 'b':
 	{
 	  uintmax_t u;
@@ -616,11 +703,6 @@ decode_options (int argc, char **argv)
 	}
 	break;
 
-      case OBSOLETE_READ_FULL_RECORDS:
-	WARN ((0, 0,
-	       _("Obsolete option name replaced by --read-full-records")));
-	/* Fall through.  */
-
       case 'B':
 	/* Try to reblock input records.  For reading 4.2BSD pipes.  */
 
@@ -629,7 +711,7 @@ decode_options (int argc, char **argv)
 	   BSD-derived systems.  This is a consequence of the block/record
 	   terminology confusion.  */
 
-	read_full_records_option = 1;
+	read_full_records_option = true;
 	break;
 
       case 'c':
@@ -661,12 +743,12 @@ decode_options (int argc, char **argv)
 	   script at the end of each tape.  */
 
 	info_script_option = optarg;
-	multi_volume_option = 1;
+	multi_volume_option = true;
 	break;
 
       case 'g':
 	listed_incremental_option = optarg;
-	after_date_option = 1;
+	after_date_option = true;
 	/* Fall through.  */
 
       case 'G':
@@ -674,13 +756,12 @@ decode_options (int argc, char **argv)
 	   directories at the beginning of the archive, and include in each
 	   directory its contents.  */
 
-	incremental_option = 1;
+	incremental_option = true;
 	break;
 
       case 'h':
 	/* Follow symbolic links.  */
-
-	dereference_option = 1;
+	dereference_option = true;
 	break;
 
       case 'i':
@@ -688,7 +769,7 @@ decode_options (int argc, char **argv)
 	   because Unix tar writes two blocks of zeros, then pads out
 	   the record with garbage.  */
 
-	ignore_zeros_option = 1;
+	ignore_zeros_option = true;
 	break;
 
       case 'I':
@@ -707,7 +788,7 @@ decode_options (int argc, char **argv)
 	break;
 
       case 'K':
-	starting_file_option = 1;
+	starting_file_option = true;
 	addname (optarg, 0);
 	break;
 
@@ -715,7 +796,7 @@ decode_options (int argc, char **argv)
 	/* When dumping directories, don't dump files/subdirectories
 	   that are on other filesystems.  */
 
-	one_file_system_option = 1;
+	one_file_system_option = true;
 	break;
 
       case 'L':
@@ -725,32 +806,28 @@ decode_options (int argc, char **argv)
 	    USAGE_ERROR ((0, 0, "%s: %s", quotearg_colon (optarg),
 			  _("Invalid tape length")));
 	  tape_length_option = 1024 * (tarlong) u;
-	  multi_volume_option = 1;
+	  multi_volume_option = true;
 	}
 	break;
 
-      case OBSOLETE_TOUCH:
-	WARN ((0, 0, _("Obsolete option name replaced by --touch")));
-	/* Fall through.  */
-
       case 'm':
-	touch_option = 1;
+	touch_option = true;
 	break;
 
       case 'M':
 	/* Make multivolume archive: when we can't write any more into
 	   the archive, re-open it, and continue writing.  */
 
-	multi_volume_option = 1;
+	multi_volume_option = true;
 	break;
 
 #if !MSDOS
       case 'N':
-	after_date_option = 1;
+	after_date_option = true;
 	/* Fall through.  */
 
       case NEWER_MTIME_OPTION:
-	if (newer_mtime_option != TYPE_MINIMUM (time_t))
+	if (NEWER_OPTION_INITIALIZED (newer_mtime_option))
 	  USAGE_ERROR ((0, 0, _("More than one threshold date")));
 
 	if (FILESYSTEM_PREFIX_LEN (optarg) != 0
@@ -763,50 +840,44 @@ decode_options (int argc, char **argv)
 		stat_error (optarg);
 		USAGE_ERROR ((0, 0, _("Date file not found")));
 	      }
-	    newer_mtime_option = st.st_mtime;
+	    newer_mtime_option.tv_sec = st.st_mtime;
+	    newer_mtime_option.tv_nsec = TIMESPEC_NS (st.st_mtim);
 	  }
 	else
 	  {
-	    newer_mtime_option = get_date (optarg, 0);
-	    if (newer_mtime_option == (time_t) -1)
-	      WARN ((0, 0, _("Substituting %s for unknown date format %s"),
-		     tartime (newer_mtime_option), quote (optarg)));
+	    if (! get_date (&newer_mtime_option, optarg, NULL))
+	      {
+		WARN ((0, 0, _("Substituting %s for unknown date format %s"),
+		       tartime (newer_mtime_option.tv_sec), quote (optarg)));
+		newer_mtime_option.tv_nsec = 0;
+	      }
+	    else
+	      textual_date_option = optarg;
 	  }
 
 	break;
 #endif /* not MSDOS */
 
       case 'o':
-	if (archive_format == DEFAULT_FORMAT)
-	  archive_format = V7_FORMAT;
-	else if (archive_format != V7_FORMAT)
-	  USAGE_ERROR ((0, 0, _("Conflicting archive format options")));
+	o_option = true;
 	break;
 
       case 'O':
-	to_stdout_option = 1;
+	to_stdout_option = true;
 	break;
 
       case 'p':
-	same_permissions_option = 1;
+	same_permissions_option = true;
 	break;
 
-      case OBSOLETE_ABSOLUTE_NAMES:
-	WARN ((0, 0, _("Obsolete option name replaced by --absolute-names")));
-	/* Fall through.  */
-
       case 'P':
-	absolute_names_option = 1;
+	absolute_names_option = true;
 	break;
 
       case 'r':
 	set_subcommand_option (APPEND_SUBCOMMAND);
 	break;
 
-      case OBSOLETE_BLOCK_NUMBER:
-	WARN ((0, 0, _("Obsolete option name replaced by --block-number")));
-	/* Fall through.  */
-
       case 'R':
 	/* Print block numbers for debugging bad tar archives.  */
 
@@ -815,17 +886,17 @@ decode_options (int argc, char **argv)
 	   BSD-derived systems.  This is a consequence of the block/record
 	   terminology confusion.  */
 
-	block_number_option = 1;
+	block_number_option = true;
 	break;
 
       case 's':
 	/* Names to extr are sorted.  */
 
-	same_order_option = 1;
+	same_order_option = true;
 	break;
 
       case 'S':
-	sparse_option = 1;
+	sparse_option = true;
 	break;
 
       case 't':
@@ -845,6 +916,10 @@ decode_options (int argc, char **argv)
 	old_files_option = UNLINK_FIRST_OLD_FILES;
 	break;
 
+      case UTC_OPTION:
+	utc_option = true;
+	break;
+
       case 'v':
 	verbose_option++;
 	break;
@@ -854,11 +929,11 @@ decode_options (int argc, char **argv)
 	break;
 
       case 'w':
-	interactive_option = 1;
+	interactive_option = true;
 	break;
 
       case 'W':
-	verify_option = 1;
+	verify_option = true;
 	break;
 
       case 'x':
@@ -889,16 +964,20 @@ decode_options (int argc, char **argv)
 	set_use_compress_program_option ("compress");
 	break;
 
-      case OBSOLETE_VERSION_CONTROL:
-	WARN ((0, 0, _("Obsolete option name replaced by --backup")));
-	/* Fall through.  */
-
       case ANCHORED_OPTION:
 	exclude_options |= EXCLUDE_ANCHORED;
 	break;
 
+      case ATIME_PRESERVE_OPTION:
+	atime_preserve_option = true;
+	break;
+
+      case CHECKPOINT_OPTION:
+	checkpoint_option = true;
+	break;
+
       case BACKUP_OPTION:
-	backup_option = 1;
+	backup_option = true;
 	if (optarg)
 	  version_control_string = optarg;
 	break;
@@ -911,10 +990,30 @@ decode_options (int argc, char **argv)
 	add_exclude (excluded, optarg, exclude_options | recursion_option);
 	break;
 
+      case FORCE_LOCAL_OPTION:
+	force_local_option = true;
+	break;
+
+      case FORMAT_OPTION:
+	set_archive_format (optarg);
+	break;
+
+      case INDEX_FILE_OPTION:
+	index_file_name = optarg;
+	break;
+
       case IGNORE_CASE_OPTION:
 	exclude_options |= FNM_CASEFOLD;
 	break;
 
+      case IGNORE_FAILED_READ_OPTION:
+	ignore_failed_read_option = true;
+	break;
+
+      case KEEP_NEWER_FILES_OPTION:
+	old_files_option = KEEP_NEWER_FILES;
+	break;
+
       case GROUP_OPTION:
 	if (! (strlen (optarg) < GNAME_FIELD_SIZE
 	       && gname_to_gid (optarg, &group_option)))
@@ -947,6 +1046,10 @@ decode_options (int argc, char **argv)
 	exclude_options &= ~ FNM_CASEFOLD;
 	break;
 
+      case NO_OVERWRITE_DIR_OPTION:
+	old_files_option = NO_OVERWRITE_DIR_OLD_FILES;
+	break;
+
       case NO_WILDCARDS_OPTION:
 	exclude_options &= ~ EXCLUDE_WILDCARDS;
 	break;
@@ -959,12 +1062,26 @@ decode_options (int argc, char **argv)
 	filename_terminator = '\0';
 	break;
 
-      case OVERWRITE_OPTION:
-	old_files_option = OVERWRITE_OLD_FILES;
+      case NUMERIC_OWNER_OPTION:
+	numeric_owner_option = true;
 	break;
 
-      case OVERWRITE_DIR_OPTION:
-	old_files_option = OVERWRITE_OLD_DIRS;
+      case OCCURRENCE_OPTION:
+	if (!optarg)
+	  occurrence_option = 1;
+	else
+	  {
+	    uintmax_t u;
+	    if (xstrtoumax (optarg, 0, 10, &u, "") == LONGINT_OK)
+	      occurrence_option = u;
+	    else
+	      FATAL_ERROR ((0, 0, "%s: %s", quotearg_colon (optarg),
+			    _("Invalid number")));
+	  }
+	break;
+
+      case OVERWRITE_OPTION:
+	old_files_option = OVERWRITE_OLD_FILES;
 	break;
 
       case OWNER_OPTION:
@@ -981,23 +1098,18 @@ decode_options (int argc, char **argv)
 	  }
 	break;
 
+      case PAX_OPTION:
+	pax_option++;
+	xheader_set_option (optarg);
+	break;
+
       case POSIX_OPTION:
-#if OLDGNU_COMPATIBILITY
-	if (archive_format == DEFAULT_FORMAT)
-	  archive_format = GNU_FORMAT;
-	else if (archive_format != GNU_FORMAT)
-	  USAGE_ERROR ((0, 0, _("Conflicting archive format options")));
-#else
-	if (archive_format == DEFAULT_FORMAT)
-	  archive_format = POSIX_FORMAT;
-	else if (archive_format != POSIX_FORMAT)
-	  USAGE_ERROR ((0, 0, _("Conflicting archive format options")));
-#endif
+	set_archive_format ("posix");
 	break;
 
       case PRESERVE_OPTION:
-	same_permissions_option = 1;
-	same_order_option = 1;
+	same_permissions_option = true;
+	same_order_option = true;
 	break;
 
       case RECORD_SIZE_OPTION:
@@ -1015,15 +1127,48 @@ decode_options (int argc, char **argv)
 	}
 	break;
 
+      case RECURSIVE_UNLINK_OPTION:
+	recursive_unlink_option = true;
+	break;
+
+      case REMOVE_FILES_OPTION:
+	remove_files_option = true;
+	break;
+
+      case RMT_COMMAND_OPTION:
+	rmt_command_option = optarg;
+	break;
+	
       case RSH_COMMAND_OPTION:
 	rsh_command_option = optarg;
 	break;
 
+      case SHOW_DEFAULTS_OPTION:
+	printf ("--format=%s -f%s -b%d\n",
+		archive_format_string (DEFAULT_ARCHIVE_FORMAT),
+		DEFAULT_ARCHIVE, DEFAULT_BLOCKING);
+	exit(0);
+
+      case STRIP_PATH_OPTION:
+	{
+	  uintmax_t u;
+	  if (! (xstrtoumax (optarg, 0, 10, &u, "") == LONGINT_OK
+		 && u == (size_t) u))
+	    USAGE_ERROR ((0, 0, "%s: %s", quotearg_colon (optarg),
+			  _("Invalid number of elements")));
+	  strip_path_elements = u;
+	}
+	break;
+
       case SUFFIX_OPTION:
-	backup_option = 1;
+	backup_option = true;
 	backup_suffix_string = optarg;
 	break;
 
+      case TOTALS_OPTION:
+	totals_option = true;
+	break;
+
       case USE_COMPRESS_PROGRAM_OPTION:
 	set_use_compress_program_option (optarg);
 	break;
@@ -1104,10 +1249,7 @@ decode_options (int argc, char **argv)
 		xrealloc (archive_name_array,
 			  sizeof (const char *) * allocated_archive_names);
 	    }
-	  archive_name_array[archive_names++] = buf;
-
-	  /* FIXME: How comes this works for many archives when buf is
-	     not xstrdup'ed?  */
+	  archive_name_array[archive_names++] = strdup (buf);
 	}
 	break;
 
@@ -1119,6 +1261,29 @@ decode_options (int argc, char **argv)
 #endif /* not DEVICE_PREFIX */
       }
 
+  /* Special handling for 'o' option:
+
+     GNU tar used to say "output old format".
+     UNIX98 tar says don't chown files after extracting (we use
+     "--no-same-owner" for this).
+
+     The old GNU tar semantics is retained when used with --create
+     option, otherwise UNIX98 semantics is assumed */
+
+  if (o_option)
+    {
+      if (subcommand_option == CREATE_SUBCOMMAND)
+	{
+	  /* GNU Tar <= 1.13 compatibility */
+	  set_archive_format ("v7");
+	}
+      else
+	{
+	  /* UNIX98 compatibility */
+	  same_owner_option = 1;
+	}
+    }
+
   /* Handle operands after any "--" argument.  */
   for (; optind < argc; optind++)
     {
@@ -1130,8 +1295,8 @@ decode_options (int argc, char **argv)
 
   if (show_version)
     {
-      printf ("tar (GNU %s) %s\n", PACKAGE, VERSION);
-      print_copyright ("2001 Free Software Foundation, Inc.");
+      printf ("tar (%s) %s\n%s\n", PACKAGE_NAME, PACKAGE_VERSION,
+	      "Copyright (C) 2004 Free Software Foundation, Inc.");
       puts (_("\
 This program comes with NO WARRANTY, to the extent permitted by law.\n\
 You may redistribute it under the terms of the GNU General Public License;\n\
@@ -1149,21 +1314,37 @@ see the file named COPYING for details."));
 
   if (archive_format == DEFAULT_FORMAT)
     {
-#if OLDGNU_COMPATIBILITY
-      archive_format = OLDGNU_FORMAT;
-#else
-      archive_format = GNU_FORMAT;
-#endif
+      if (pax_option)
+	archive_format = POSIX_FORMAT;
+      else
+	archive_format = DEFAULT_ARCHIVE_FORMAT;
     }
 
-  if (archive_format == GNU_FORMAT && getenv ("POSIXLY_CORRECT"))
-    archive_format = POSIX_FORMAT;
+  if (volume_label_option && subcommand_option == CREATE_SUBCOMMAND)
+    assert_format (FORMAT_MASK (OLDGNU_FORMAT)
+		   | FORMAT_MASK (GNU_FORMAT));
 
-  if ((volume_label_option
-       || incremental_option || multi_volume_option || sparse_option)
-      && archive_format != OLDGNU_FORMAT && archive_format != GNU_FORMAT)
-    USAGE_ERROR ((0, 0,
-		  _("GNU features wanted on incompatible archive format")));
+
+  if (incremental_option || multi_volume_option)
+    assert_format (FORMAT_MASK (OLDGNU_FORMAT) | FORMAT_MASK (GNU_FORMAT));
+
+  if (sparse_option)
+    assert_format (FORMAT_MASK (OLDGNU_FORMAT)
+		   | FORMAT_MASK (GNU_FORMAT)
+		   | FORMAT_MASK (POSIX_FORMAT));
+
+  if (occurrence_option)
+    {
+      if (!input_files && !files_from_option)
+	USAGE_ERROR ((0, 0,
+		      _("--occurrence is meaningless without a file list")));
+      if (subcommand_option != DELETE_SUBCOMMAND
+	  && subcommand_option != DIFF_SUBCOMMAND
+	  && subcommand_option != EXTRACT_SUBCOMMAND
+	  && subcommand_option != LIST_SUBCOMMAND)
+	    USAGE_ERROR ((0, 0,
+			  _("--occurrence cannot be used in the requested operation mode")));
+    }
 
   if (archive_names == 0)
     {
@@ -1180,10 +1361,10 @@ see the file named COPYING for details."));
 
   if (archive_names > 1 && !multi_volume_option)
     USAGE_ERROR ((0, 0,
-		  _("Multiple archive files requires `-M' option")));
+		  _("Multiple archive files require `-M' option")));
 
   if (listed_incremental_option
-      && newer_mtime_option != TYPE_MINIMUM (time_t))
+      && NEWER_OPTION_INITIALIZED (newer_mtime_option))
     USAGE_ERROR ((0, 0,
 		  _("Cannot combine --listed-incremental with --newer")));
 
@@ -1200,15 +1381,50 @@ see the file named COPYING for details."));
 	    : 0));
       if (volume_label_max_len < strlen (volume_label_option))
 	USAGE_ERROR ((0, 0,
-		      _("%s: Volume label is too long (limit is %lu bytes)"),
+		      ngettext ("%s: Volume label is too long (limit is %lu byte)",
+				"%s: Volume label is too long (limit is %lu bytes)",
+				volume_label_max_len),
 		      quotearg_colon (volume_label_option),
 		      (unsigned long) volume_label_max_len));
     }
 
+  if (verify_option)
+    {
+      if (multi_volume_option)
+	USAGE_ERROR ((0, 0, _("Cannot verify multi-volume archives")));
+      if (use_compress_program_option)
+	USAGE_ERROR ((0, 0, _("Cannot verify compressed archives")));
+    }
+
+  if (use_compress_program_option)
+    {
+      if (multi_volume_option)
+	USAGE_ERROR ((0, 0, _("Cannot use multi-volume compressed archives")));
+      if (subcommand_option == UPDATE_SUBCOMMAND)
+	USAGE_ERROR ((0, 0, _("Cannot update compressed archives")));
+    }
+
+  /* It is no harm to use --pax-option on non-pax archives in archive
+     reading mode. It may even be useful, since it allows to override
+     file attributes from tar headers. Therefore I allow such usage.
+     --gray */
+  if (pax_option
+      && archive_format != POSIX_FORMAT
+      && (subcommand_option != EXTRACT_SUBCOMMAND
+	  || subcommand_option != DIFF_SUBCOMMAND
+	  || subcommand_option != LIST_SUBCOMMAND))
+    USAGE_ERROR ((0, 0, _("--pax-option can be used only on POSIX archives")));
+
   /* If ready to unlink hierarchies, so we are for simpler files.  */
   if (recursive_unlink_option)
     old_files_option = UNLINK_FIRST_OLD_FILES;
 
+  if (utc_option)
+    verbose_option = 2;
+
+  if (!rmt_command_option)
+    rmt_command_option = DEFAULT_RMT_COMMAND;
+  
   /* Forbid using -c with no input files whatsoever.  Check that `-f -',
      explicit or implied, is used correctly.  */
 
@@ -1253,7 +1469,18 @@ see the file named COPYING for details."));
 
   if (backup_option)
     backup_type = xget_version ("--backup", version_control_string);
+
+  if (verbose_option && textual_date_option)
+    {
+      /* FIXME: tartime should support nanoseconds, too, so that this
+	 comparison doesn't complain about lost nanoseconds.  */
+      char const *treated_as = tartime (newer_mtime_option.tv_sec);
+      if (strcmp (textual_date_option, treated_as) != 0)
+	WARN ((0, 0, _("Treating date `%s' as %s + %ld nanoseconds"),
+	       textual_date_option, treated_as, newer_mtime_option.tv_nsec));
+    }
 }
+
 
 /* Tar proper.  */
 
@@ -1291,7 +1518,7 @@ main (int argc, char **argv)
   /* Decode options.  */
 
   decode_options (argc, argv);
-  name_init (argc, argv);
+  name_init ();
 
   /* Main command execution.  */
 
@@ -1325,7 +1552,11 @@ main (int argc, char **argv)
     case EXTRACT_SUBCOMMAND:
       extr_init ();
       read_and (extract_archive);
+
+      /* FIXME: should extract_finish () even if an ordinary signal is
+	 received.  */
       extract_finish ();
+
       break;
 
     case LIST_SUBCOMMAND:
@@ -1338,6 +1569,9 @@ main (int argc, char **argv)
       break;
     }
 
+  if (check_links_option)
+      check_links ();
+
   if (volno_file_option)
     closeout_volume_number ();
 
@@ -1346,9 +1580,29 @@ main (int argc, char **argv)
   free (archive_name_array);
   name_term ();
 
-  if (stdlis == stdout && (ferror (stdout) || fclose (stdout) != 0))
+  if (stdlis != stderr && (ferror (stdlis) || fclose (stdlis) != 0))
     FATAL_ERROR ((0, 0, _("Error in writing to standard output")));
   if (exit_status == TAREXIT_FAILURE)
     error (0, 0, _("Error exit delayed from previous errors"));
-  exit (exit_status);
+  if (ferror (stderr) || fclose (stderr) != 0)
+    exit_status = TAREXIT_FAILURE;
+  return exit_status;
+}
+
+void
+tar_stat_init (struct tar_stat_info *st)
+{
+  memset (st, 0, sizeof (*st));
+}
+
+void
+tar_stat_destroy (struct tar_stat_info *st)
+{
+  free (st->orig_file_name);
+  free (st->file_name);
+  free (st->link_name);
+  free (st->uname);
+  free (st->gname);
+  free (st->sparse_map);
+  memset (st, 0, sizeof (*st));
 }
